
from glob import glob
import os
from pathlib import Path
from PIL.Image import Image, open, fromarray
import numpy as np


DEFAULT_PATH = Path('..', '..', 'data')
VALID_FILE_TYPES = ["jpg", "jpeg", "png"]


def image_name(img):
    return Path(img.filename).name


def empty_image(img):
    return fromarray(np.full(img.size[::-1], 0))


class Dataset():
    def __init__(self, data_path=DEFAULT_PATH):
        self.images_path = data_path / 'images'
        self.labels_path = data_path / 'labels'

        _images = [glob(f'{self.images_path}/*.{ext}') for ext in VALID_FILE_TYPES]
        self.images = [open(item) for i in _images for item in i]

        if len(self.images) == 0:
            raise Exception('No images were found')

    def get(self, index: int) -> tuple[Image, str]:
        index %= len(self.images)
        image = self.images[index]
        return image, image_name(image)

    def get_labels_for(self, image):
        try:
            return open(f'{self.labels_path}/{image_name(image)}')
        except IOError:
            return empty_image(image)

    def save_labels(self, index: int, data):
        im = fromarray(data.astype(np.uint8))
        _, name = self.get(index)
        save_path = f'{self.labels_path}/{name}'

        if np.sum(im) > 0:
            im.save(save_path)
        elif os.path.exists(save_path):
            os.remove(save_path)



import napari
from napari import Viewer as NapariViewer, layers
import numpy as np

from dataset_viewer.dataset import Dataset


class Viewer():
    __image_idx = 0

    def __init__(self, dataset = Dataset()):
        self.viewer = NapariViewer()
        self.dataset = dataset

        self.labels_layer = None
        self.__set_bindings()

    def start(self):
        self.__show_image(0)
        napari.run()

    def __set_bindings(self):
        # Add reset option ('r')
        self.__bind_key('l', self.__current_label_name)
        self.__bind_key('Escape', self.__exit)
        self.__bind_key('Left', self.__previous)
        self.__bind_key('Right', self.__next)

    def __bind_key(self, key, func):
        self.viewer.bind_key(key, func)

    def __previous(self, viewer: NapariViewer):
        self.__show_image(-1)

    def __next(self, viewer: NapariViewer):
        self.__show_image(1)

    def __show_image(self, move=0):
        if self.labels_layer is not None:
            self.dataset.save_labels(self.__image_idx, self.labels_layer.data)

        self.__image_idx += move
        self.viewer.layers.clear()

        image, name = self.dataset.get(self.__image_idx)
        self.viewer.add_image(np.array(image), name=name)
        self.__set_labels_layer(image)

    def __set_labels_layer(self, image):
        labels = self.dataset.get_labels_for(image)
        self.labels_layer = self.viewer.add_labels(np.array(labels), name='segmentation')
        self.labels_layer.mode = 'PAINT'

    def __current_label_name(self, viewer: NapariViewer):
        active_layer = viewer.layers.selection.active

        if active_layer is not None:
            print(viewer.layers.selection.active.name)

    def __exit(self, viewer: NapariViewer):
        viewer.close()



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='dataset-viewer',
    long_description=long_description,
    long_description_content_type='text/markdown'
)